DS_SHORT_NAME_MAPPING = {
    # MetaWorld
    "aliangdw_metaworld_metaworld_train": "Metaworld (Train)",
    "aliangdw_metaworld_metaworld_eval": "Metaworld (Eval)",
    # LIBERO
    "abraranwar_libero_rfm_libero256_10": "Libero (10)",
    "abraranwar_libero_rfm_libero256_90": "Libero (90)",
    "abraranwar_libero_rfm_libero256_object": "Libero (Object)",
    "abraranwar_libero_rfm_libero256_spatial": "Libero (Spatial)",
    "abraranwar_libero_rfm_libero256_goal": "Libero (Goal)",
    # LIBERO Failure
    "ykorkmaz_libero_failure_rfm_libero_10_failure": "Libero (10) Failure",
    "ykorkmaz_libero_failure_rfm_libero_90_failure": "Libero (90) Failure",
    "ykorkmaz_libero_failure_rfm_libero_object_failure": "Libero (Object) Failure",
    "ykorkmaz_libero_failure_rfm_libero_spatial_failure": "Libero (Spatial) Failure",
    "ykorkmaz_libero_failure_rfm_libero_goal_failure": "Libero (Goal) Failure",
    # OXE Train
    "jesbu1_oxe_rfm_oxe_aloha_mobile": "OXE (Aloha Mobile)",
    "jesbu1_oxe_rfm_oxe_austin_buds_dataset_converted_externally_to_rlds": "oxe_austin_buds",
    "jesbu1_oxe_rfm_oxe_bc_z": "OXE (BC-Z)",
    "jesbu1_oxe_rfm_oxe_berkeley_cable_routing": "oxe_berkeley_cable",
    "jesbu1_oxe_rfm_oxe_berkeley_fanuc_manipulation": "OXE (Berkeley Fanuc Manipulation)",
    "jesbu1_oxe_rfm_oxe_berkeley_mvp_converted_externally_to_rlds": "OXE (Berkeley MVP)",
    "jesbu1_oxe_rfm_oxe_berkeley_rpt_converted_externally_to_rlds": "OXE (Berkeley RPT)",
    "jesbu1_oxe_rfm_oxe_bridge_v2": "OXE (Bridge V2)",
    "jesbu1_oxe_rfm_oxe_dlr_edan_shared_control_converted_externally_to_rlds": "OXE (DLR EDAN Shared Control)",
    "jesbu1_oxe_rfm_oxe_droid": "OXE (Droid)",
    "jesbu1_oxe_rfm_oxe_fractal20220817_data": "OXE (Fractal 20220817 Data)",
    "jesbu1_oxe_rfm_oxe_furniture_bench_dataset_converted_externally_to_rlds": "oxe_furniture_bench",
    "jesbu1_oxe_rfm_oxe_iamlab_cmu_pickup_insert_converted_externally_to_rlds": "OXE (CMU Pickup Insert)",
    "jesbu1_oxe_rfm_oxe_imperialcollege_sawyer_wrist_cam": "oxe_imperial_sawyer",
    "jesbu1_oxe_rfm_oxe_language_table": "OXE (Language Table)",
    "jesbu1_oxe_rfm_oxe_jaco_play": "OXE (Jaco Play)",
    "jesbu1_oxe_rfm_oxe_nyu_rot_dataset_converted_externally_to_rlds": "OXE (NYU Rot Dataset)",
    "jesbu1_oxe_rfm_oxe_robo_set": "OXE (Robo Set)",
    "jesbu1_oxe_rfm_oxe_stanford_hydra_dataset_converted_externally_to_rlds": "OXE (Stanford Hydra)",
    "jesbu1_oxe_rfm_oxe_tokyo_u_lsmo_converted_externally_to_rlds": "OXE (Tokyo U LSMo)",
    "jesbu1_oxe_rfm_oxe_toto": "OXE (Toto)",
    "jesbu1_oxe_rfm_oxe_ucsd_kitchen_dataset_converted_externally_to_rlds": "OXE (UCSD Kitchen Dataset)",
    "jesbu1_oxe_rfm_oxe_utaustin_mutex": "OXE (UT Austin Mutex)",
    # OXE Eval
    "jesbu1_oxe_rfm_eval_oxe_bc_z_eval": "OXE (BC-Z Eval)",
    "jesbu1_oxe_rfm_eval_oxe_berkeley_cable_routing_eval": "OXE (Berkeley Cable Routing Eval)",
    "jesbu1_oxe_rfm_eval_oxe_bridge_v2_eval": "OXE (Bridge V2 Eval)",
    "jesbu1_oxe_rfm_eval_oxe_jaco_play_eval": "OXE (Jaco Play Eval)",
    "jesbu1_oxe_rfm_eval_oxe_toto_eval": "OXE (Toto Eval)",
    "jesbu1_oxe_rfm_eval_oxe_viola_eval": "OXE (Viola Eval)",
    # Others - Train
    "jesbu1_molmoact_rfm_molmoact_dataset_household": "Molmoact (Household)",
    "jesbu1_molmoact_rfm_molmoact_dataset_tabletop": "molmoact_tabletop",
    "jesbu1_egodex_rfm_egodex_part1": "Egodex (Part 1)",
    "jesbu1_egodex_rfm_egodex_part2": "Egodex (Part 2)",
    "jesbu1_egodex_rfm_egodex_part3": "Egodex (Part 3)",
    "jesbu1_egodex_rfm_egodex_part4": "Egodex (Part 4)",
    "jesbu1_egodex_rfm_egodex_part5": "Egodex (Part 5)",
    "abraranwar_agibotworld_alpha_rfm_agibotworld": "Agibotworld",
    "jesbu1_epic_rfm_epic": "epic_kitchens",
    "jesbu1_galaxea_rfm_galaxea_part1_r1_lite": "Galaxea (Part 1)",
    "jesbu1_galaxea_rfm_galaxea_part2_r1_lite": "Galaxea (Part 2)",
    "jesbu1_galaxea_rfm_galaxea_part3_r1_lite": "Galaxea (Part 3)",
    "jesbu1_galaxea_rfm_galaxea_part4_r1_lite": "Galaxea (Part 4)",
    "jesbu1_galaxea_rfm_galaxea_part5_r1_lite": "Galaxea (Part 5)",
    "jesbu1_humanoid_everyday_rfm_humanoid_everyday_rfm": "Humanoid Everyday",
    # Others - Eval
    "jesbu1_egodex_rfm_egodex_test": "Egodex (Test)",
    # Paired
    "jesbu1_h2r_rfm_h2r": "H2R",
    "jesbu1_motif_rfm_motif_rfm": "Motif",
    "anqil_rh20t_subset_rfm_rh20t_human": "Rh20t (Human)",
    "anqil_rh20t_subset_rfm_rh20t_robot": "Rh20t (Robot)",
    "jesbu1_ph2d_rfm_ph2d": "Ph2d",
    "aliangdw_utd_so101_human_utd_so101_human": "Utd SO101 (Human)",
    "aliangdw_utd_so101_policy_ranking_utd_so101_policy_ranking": "Utd SO101 (Policy Ranking)",
    # Suboptimal/Fail
    "jesbu1_roboarena_0825_rfm_roboarena": "Roboarena",
    "jesbu1_fino_net_rfm_fino_net": "Fino Net",
    "jesbu1_failsafe_rfm_failsafe": "Failsafe",
    "jesbu1_soar_rfm_soar_rfm": "Soar",
    "jesbu1_auto_eval_rfm_auto_eval_rfm": "Auto Eval",
    "jesbu1_racer_rfm_racer_train": "Racer (Train)",
    "jesbu1_racer_rfm_racer_val": "Racer (Val)",
    "jesbu1_roboarena_eval_debug_nowrist_roboarena_eval_debug_nowrist": "Roboarena (Eval Debug No Wrist)",
    "jesbu1_roboarena_0825_rfm_roboarena": "Roboarena (W Wrist)",
    # Policy Ranking
    "aliangdw_usc_xarm_policy_ranking_usc_xarm_policy_ranking": "Usc Xarm (Policy Ranking)",
    "aliangdw_usc_franka_policy_ranking_usc_franka_policy_ranking": "Usc Franka (Policy Ranking)",
    "jesbu1_mit_franka_p-rank_rfm_mit_franka_p-rank_rfm": "Mit Franka (P-Rank)",
    "jesbu1_utd_so101_clean_policy_ranking_top_utd_so101_clean_policy_ranking_top": "Utd SO101 (Clean Policy Ranking Top)",
    "jesbu1_utd_so101_clean_policy_ranking_wrist_utd_so101_clean_policy_ranking_wrist": "Utd SO101 (Clean Policy Ranking Wrist)",
    "jesbu1_usc_koch_p_ranking_rfm_usc_koch_p_ranking_all": "Usc Koch (P-Rank All)",
    # Reward Alignment
    "abraranwar_usc_koch_rewind_rfm_usc_koch_rewind": "Usc Koch Rewind",
    # Extras
    "jesbu1_roboreward_rfm_roboreward_test": "RoboReward (Test)",
    "jesbu1_roboreward_rfm_roboreward_val": "RoboReward (Val)",
}
