DS_SHORT_NAME_MAPPING = {
    # MetaWorld
    "aliangdw_metaworld_metaworld_train": "mw_train",
    "aliangdw_metaworld_metaworld_eval": "mw_eval",
    # LIBERO
    "abraranwar_libero_rfm_libero256_10": "libero_10",
    "abraranwar_libero_rfm_libero256_90": "libero_90",
    "abraranwar_libero_rfm_libero256_object": "libero_object",
    "abraranwar_libero_rfm_libero256_spatial": "libero_spatial",
    "abraranwar_libero_rfm_libero256_goal": "libero_goal",
    # LIBERO Failure
    "ykorkmaz_libero_failure_rfm_libero_10_failure": "libero_10_failure",
    "ykorkmaz_libero_failure_rfm_libero_90_failure": "libero_90_failure",
    "ykorkmaz_libero_failure_rfm_libero_object_failure": "libero_object_failure",
    "ykorkmaz_libero_failure_rfm_libero_spatial_failure": "libero_spatial_failure",
    "ykorkmaz_libero_failure_rfm_libero_goal_failure": "libero_goal_failure",
    # OXE Train
    "jesbu1_oxe_rfm_oxe_aloha_mobile": "oxe_aloha_mobile",
    "jesbu1_oxe_rfm_oxe_austin_buds_dataset_converted_externally_to_rlds": "oxe_austin_buds",
    "jesbu1_oxe_rfm_oxe_bc_z": "oxe_bc_z",
    "jesbu1_oxe_rfm_oxe_berkeley_cable_routing": "oxe_berkeley_cable",
    "jesbu1_oxe_rfm_oxe_berkeley_fanuc_manipulation": "oxe_berkeley_fanuc",
    "jesbu1_oxe_rfm_oxe_berkeley_mvp_converted_externally_to_rlds": "oxe_berkeley_mvp",
    "jesbu1_oxe_rfm_oxe_berkeley_rpt_converted_externally_to_rlds": "oxe_berkeley_rpt",
    "jesbu1_oxe_rfm_oxe_bridge_v2": "oxe_bridge_v2",
    "jesbu1_oxe_rfm_oxe_dlr_edan_shared_control_converted_externally_to_rlds": "oxe_dlr_edan",
    "jesbu1_oxe_rfm_oxe_droid": "oxe_droid",
    "jesbu1_oxe_rfm_oxe_fractal20220817_data": "oxe_fractal",
    "jesbu1_oxe_rfm_oxe_furniture_bench_dataset_converted_externally_to_rlds": "oxe_furniture_bench",
    "jesbu1_oxe_rfm_oxe_iamlab_cmu_pickup_insert_converted_externally_to_rlds": "oxe_cmu_pickup",
    "jesbu1_oxe_rfm_oxe_imperialcollege_sawyer_wrist_cam": "oxe_imperial_sawyer",
    "jesbu1_oxe_rfm_oxe_language_table": "oxe_language_table",
    "jesbu1_oxe_rfm_oxe_jaco_play": "oxe_jaco",
    "jesbu1_oxe_rfm_oxe_nyu_rot_dataset_converted_externally_to_rlds": "oxe_nyu_rot",
    "jesbu1_oxe_rfm_oxe_robo_set": "oxe_robo_set",
    "jesbu1_oxe_rfm_oxe_stanford_hydra_dataset_converted_externally_to_rlds": "oxe_stanford_hydra",
    "jesbu1_oxe_rfm_oxe_tokyo_u_lsmo_converted_externally_to_rlds": "oxe_tokyo_lsmo",
    "jesbu1_oxe_rfm_oxe_toto": "oxe_toto",
    "jesbu1_oxe_rfm_oxe_ucsd_kitchen_dataset_converted_externally_to_rlds": "oxe_ucsd_kitchen",
    "jesbu1_oxe_rfm_oxe_utaustin_mutex": "oxe_utaustin_mutex",
    # OXE Eval
    "jesbu1_oxe_rfm_eval_oxe_bc_z_eval": "oxe_bc_z_eval",
    "jesbu1_oxe_rfm_eval_oxe_berkeley_cable_routing_eval": "oxe_berkeley_cable_eval",
    "jesbu1_oxe_rfm_eval_oxe_bridge_v2_eval": "oxe_bridge_v2_eval",
    "jesbu1_oxe_rfm_eval_oxe_jaco_play_eval": "oxe_jaco_eval",
    "jesbu1_oxe_rfm_eval_oxe_toto_eval": "oxe_toto_eval",
    "jesbu1_oxe_rfm_eval_oxe_viola_eval": "oxe_viola_eval",
    # Others - Train
    "jesbu1_molmoact_rfm_molmoact_dataset_household": "molmoact_household",
    "jesbu1_molmoact_rfm_molmoact_dataset_tabletop": "molmoact_tabletop",
    "jesbu1_egodex_rfm_egodex_part1": "egodex_part1",
    "jesbu1_egodex_rfm_egodex_part2": "egodex_part2",
    "jesbu1_egodex_rfm_egodex_part3": "egodex_part3",
    "jesbu1_egodex_rfm_egodex_part4": "egodex_part4",
    "jesbu1_egodex_rfm_egodex_part5": "egodex_part5",
    "abraranwar_agibotworld_alpha_rfm_agibotworld": "agibotworld",
    "jesbu1_epic_rfm_epic": "epic_kitchens",
    "jesbu1_galaxea_rfm_galaxea_part1_r1_lite": "galaxea_part1",
    "jesbu1_galaxea_rfm_galaxea_part2_r1_lite": "galaxea_part2",
    "jesbu1_galaxea_rfm_galaxea_part3_r1_lite": "galaxea_part3",
    "jesbu1_galaxea_rfm_galaxea_part4_r1_lite": "galaxea_part4",
    "jesbu1_galaxea_rfm_galaxea_part5_r1_lite": "galaxea_part5",
    "jesbu1_humanoid_everyday_rfm_humanoid_everyday_rfm": "humanoid_everyday",
    # Others - Eval
    "jesbu1_egodex_rfm_egodex_test": "egodex_test",
    # Paired
    "jesbu1_h2r_rfm_h2r": "h2r",
    "jesbu1_motif_rfm_motif_rfm": "motif",
    "anqil_rh20t_subset_rfm_rh20t_human": "rh20t",
    "anqil_rh20t_subset_rfm_rh20t_robot": "rh20t",
    "jesbu1_ph2d_rfm_ph2d": "ph2d",
    "aliangdw_utd_so101_human_utd_so101_human": "utd_so101",
    "aliangdw_utd_so101_policy_ranking_utd_so101_policy_ranking": "utd_so101",
    # Suboptimal/Fail
    "jesbu1_roboarena_0825_rfm_roboarena": "roboarena",
    "jesbu1_fino_net_rfm_fino_net": "fino_net",
    "jesbu1_failsafe_rfm_failsafe": "failsafe",
    "jesbu1_soar_rfm_soar_rfm": "soar",
    "jesbu1_auto_eval_rfm_auto_eval_rfm": "auto_eval",
    "jesbu1_racer_rfm_racer_train": "racer_train",
    "jesbu1_racer_rfm_racer_val": "racer_val",
    "jesbu1_roboarena_eval_debug_nowrist_roboarena_eval_debug_nowrist": "roboarena_eval",
    "jesbu1_roboarena_0825_rfm_roboarena": "roboarena_w_wrist",
    # Policy Ranking
    "aliangdw_usc_xarm_policy_ranking_usc_xarm_policy_ranking": "usc_xarm",
    "aliangdw_usc_franka_policy_ranking_usc_franka_policy_ranking": "usc_franka",
    "jesbu1_mit_franka_p-rank_rfm_mit_franka_p-rank_rfm": "mit_franka",
    "jesbu1_utd_so101_clean_policy_ranking_top_utd_so101_clean_policy_ranking_top": "utd_so101_clean_top",
    "jesbu1_utd_so101_clean_policy_ranking_wrist_utd_so101_clean_policy_ranking_wrist": "utd_so101_clean_wrist",
    "jesbu1_rfm_new_mit_franka_rfm_rfm_new_mit_franka_rfm": "rfm_new_mit_franka",
    "ykorkmaz_usc_trossen_rfm_usc_trossen": "usc_trossen",
    # Reward Alignment
    "abraranwar_usc_koch_rewind_rfm_usc_koch_rewind": "usc_koch_rewind",
    # RoboFAC
    "aliangdw_robofac_rbm_robofac": "robofac",
}
